@@ -3,7 +3,7 @@ use crate::algebra::abstr::Ring;
 /// Integer
 ///
 ///<a href="https://en.wikipedia.org/wiki/Integer">https://en.wikipedia.org/wiki/Integer</a>
-pub trait Integer: Ring
+pub trait Integer: Ring + Eq
 {
 }
 
