@@ -1,12 +1,11 @@
 //! Polynomial
 use std::fmt::{Result, Formatter, Display};
-use crate::algebra::abstr::{Real, AbsDiffEq, Monoid, Semigroup, SemigroupAdd, Quasigroup};
+use crate::algebra::abstr::{Real, AbsDiffEq, Monoid, Semigroup, SemigroupAdd, Quasigroup, RelativeEq};
 use crate::algebra::{
     abstr::{Field, Scalar},
 };
 use std::ops::{Add, Mul, Div, Sub, Neg, AddAssign, MulAssign, SubAssign};
 use crate::algebra::abstr::Zero;
-use crate::abs_diff_eq;
 use crate::algebra::abstr::magma::{MagmaAdd, MagmaMul, Magma};
 use crate::algebra::abstr::monoid::{MonoidAdd};
 use crate::algebra::abstr::identity::{Identity};
@@ -526,7 +525,7 @@ impl<T> MulAssign for Polynomial<T>
 }
 
 impl<'a, 'b, T> Div<&'b Polynomial<T>> for &'a Polynomial<T>
-    where T: Field + Scalar
+    where T: Field + Scalar + AbsDiffEq<Epsilon = T>
 {
     type Output = (Polynomial<T>, Polynomial<T>);
 
@@ -585,7 +584,7 @@ impl<T> Polynomial<T>
         for i in (1..len).rev()
         {
             let v = &coef[i];
-            if abs_diff_eq!(*v, T::zero())
+            if v.abs_diff_eq(&T::zero(), T::default_epsilon())
             {
                 coef.pop();
             }
@@ -604,7 +603,7 @@ impl<T> Polynomial<T>
 }
 
 impl<T> Polynomial<T>
-    where T: Field + Scalar
+    where T: Field + Scalar + AbsDiffEq<Epsilon = T>
 {
     /// Differentiate polynomial
     ///
@@ -676,24 +675,8 @@ impl<T> Polynomial<T>
     }
 }
 
-impl<T> AbsDiffEq for Polynomial<T>
-    where T: AbsDiffEq<Epsilon = T>
-{
-    type Epsilon = T;
-
-    fn default_epsilon() -> T
-    {
-        T::default_epsilon()
-    }
-
-    fn abs_diff_eq(&self, _other: &Polynomial<T>, _epsilon: T) -> bool
-    {
-       unimplemented!();
-    }
-}
-
 impl<T> Magma<Addition> for Polynomial<T>
-    where T: MagmaAdd + Magma<Addition> + Scalar + AbsDiffEq<Epsilon = T>
+    where T: MagmaAdd + Magma<Addition> + Scalar
 {
     fn operate(self, rhs: Self) -> Self
     {
@@ -702,13 +685,13 @@ impl<T> Magma<Addition> for Polynomial<T>
 }
 
 impl<T> MagmaAdd for Polynomial<T>
-    where T: MagmaAdd + Scalar + AbsDiffEq<Epsilon = T>
+    where T: MagmaAdd + Scalar
 {
 
 }
 
 impl<T> Magma<Multiplication> for Polynomial<T>
-    where T: MagmaMul + MonoidAdd + Magma<Multiplication> + Scalar + AbsDiffEq<Epsilon = T>
+    where T: MagmaMul + MonoidAdd + Magma<Multiplication> + Scalar
 {
     fn operate(self, rhs: Self) -> Self
     {
@@ -717,19 +700,19 @@ impl<T> Magma<Multiplication> for Polynomial<T>
 }
 
 impl<T> MagmaMul for Polynomial<T>
-    where T: MagmaMul + MonoidAdd + Scalar + AbsDiffEq<Epsilon = T>
+    where T: MagmaMul + MonoidAdd + Scalar
 {
 
 }
 
 impl<T> Semigroup<Addition> for Polynomial<T>
-    where T: MagmaAdd + Scalar + AbsDiffEq<Epsilon = T>
+    where T: MagmaAdd + Scalar
 {
 
 }
 
 impl<T> SemigroupAdd for Polynomial<T>
-    where T: SemigroupAdd + Semigroup<Addition> + MagmaAdd + Scalar + AbsDiffEq<Epsilon = T>
+    where T: SemigroupAdd + Semigroup<Addition> + MagmaAdd + Scalar
 {
 }
 
@@ -743,36 +726,80 @@ impl<T> Identity<Addition> for Polynomial<T>
 }
 
 impl<T> Monoid<Addition> for Polynomial<T>
-    where T: MagmaAdd + Scalar + Identity<Addition> + AbsDiffEq<Epsilon = T>
+    where T: MagmaAdd + Scalar + Identity<Addition>
 {
 }
 
 impl<T> MonoidAdd for Polynomial<T>
-    where T: Monoid<Addition> + SemigroupAdd + Zero + Scalar + AbsDiffEq<Epsilon = T>
+    where T: Monoid<Addition> + SemigroupAdd + Zero + Scalar
 {
 
 }
 
 impl<T> Quasigroup<Addition> for Polynomial<T>
-    where T: Quasigroup<Addition> + Scalar + MagmaAdd + AbsDiffEq<Epsilon = T>
+    where T: Quasigroup<Addition> + Scalar + MagmaAdd
 {
 
 }
 
 impl<T> Loop<Addition> for Polynomial<T>
-    where T: Loop<Addition> + Quasigroup<Addition> + Scalar + MagmaAdd + AbsDiffEq<Epsilon = T>
+    where T: Loop<Addition> + Quasigroup<Addition> + Scalar + MagmaAdd
 {
 
 }
 
 impl<T> Group<Addition> for Polynomial<T>
-    where T: Group<Addition> + Loop<Addition> + Quasigroup<Addition> + Scalar + MagmaAdd + AbsDiffEq<Epsilon = T>
+    where T: Group<Addition> + Loop<Addition> + Quasigroup<Addition> + Scalar + MagmaAdd
 {
 
 }
 
 impl<T> GroupAdd for Polynomial<T>
-    where T: GroupAdd + Group<Addition> + Loop<Addition> + Quasigroup<Addition> + Scalar + MagmaAdd + AbsDiffEq<Epsilon = T> + MonoidAdd
+    where T: GroupAdd + Group<Addition> + Loop<Addition> + Quasigroup<Addition> + Scalar + MagmaAdd + MonoidAdd
 {
 
+}
+
+impl<T> AbsDiffEq for Polynomial<T>
+    where T: AbsDiffEq<Epsilon = T> + Clone
+{
+    type Epsilon = T;
+
+    fn default_epsilon() -> T
+    {
+        T::default_epsilon()
+    }
+
+    fn abs_diff_eq(&self, other: &Polynomial<T>, epsilon: T) -> bool
+    {
+        for (a, b) in self.coef.iter().zip(other.coef.iter())
+        {
+            if a.abs_diff_ne(b, epsilon.clone())
+            {
+                return false;
+            }
+        }
+        return true;
+    }
+}
+
+impl<T> RelativeEq for Polynomial<T>
+    where T: RelativeEq<Epsilon = T> + Clone
+{
+    fn default_max_relative() -> T
+    {
+        T::default_epsilon()
+    }
+
+    fn relative_eq(&self, other: &Self, epsilon: T, max_relative: T) -> bool
+    {
+        for (a, b) in self.coef.iter().zip(other.coef.iter())
+        {
+            if a.relative_ne(b, epsilon.clone(), max_relative.clone())
+            {
+                return false;
+            }
+        }
+        return true;
+    }
 }
\ No newline at end of file
