@@ -1,4 +1,3 @@
-use crate::algebra::abstr::{Field, Scalar};
 use std::slice::Iter;
 
 pub struct MatrixIterator<'a, T>
@@ -15,7 +14,6 @@ impl<'a, T> MatrixIterator<'a, T>
 }
 
 impl<'a, T> Iterator for MatrixIterator<'a, T>
-    where T: Field + Scalar
 {
     type Item = &'a T;
 
