@@ -7,3 +7,7 @@ pub use self::{matrix::Matrix, vector::Vector};
 pub mod vector;
 #[macro_use]
 pub mod matrix;
+#[cfg(feature = "lapack")]
+pub mod lapack;
+#[cfg(feature = "lapack")]
+pub mod blas;
